package sqlite;

import java.sql.Connection;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;

public class Joins {
 
    public static void main(String[] args) throws Exception {
        Connection connection = SetUp3.initialize();

        Statement stat = connection.createStatement();

        Table.print(connection, "Artists");
        System.out.println();
        Table.print(connection,  "CDs");
        System.out.println();
        
        String[][] columnInfo1 = {{"ArtistID", "INTEGER"}, {"ArtistName", "TEXT"}, {"CDID", "INTEGER"}, {"ArtistID", "INTEGER"}, {"Title", "TEXT"}, {"Date", "TEXT"}};

	    try{
	        ResultSet rs = stat.executeQuery("select * from Artists CROSS JOIN CDs;");
		    System.out.println("\n\nselect * from Artists CROSS JOIN CDs; = ");
	        QueryResult.print(rs, columnInfo1);

	        rs.close();
	    }catch(SQLException sqlException) {
	        System.err.println("Could NOT Perform Query 1");
	        System.err.println(sqlException);
	    }
	    
	    String[][] columnInfo2 = {{"ArtistName", "TEXT"}, {"Title", "TEXT"}};

	    try{
	        ResultSet rs = stat.executeQuery("select Artists.ArtistName, CDs.Title from Artists CROSS JOIN CDs where Artists.ArtistID = CDs.ArtistID;");
		    System.out.println("\n\nselect Artists.ArtistName, CDs.Title from Artists CROSS JOIN CDs where Artists.ArtistID = CDs.ArtistID; = ");
	        QueryResult.print(rs, columnInfo2);

	        rs.close();
	    }catch(SQLException sqlException) {
	        System.err.println("Could NOT Perform Query 2");
	        System.err.println(sqlException);
	    }

	    try{
	        ResultSet rs = stat.executeQuery("select Artists.ArtistName, CDs.Title from Artists INNER JOIN CDs on Artists.ArtistID = CDs.ArtistID;");
		    System.out.println("\n\nselect Artists.ArtistName, CDs.Title from Artists INNER JOIN CDs on Artists.ArtistID = CDs.ArtistID; = ");
	        QueryResult.print(rs, columnInfo2);

	        rs.close();
	    }catch(SQLException sqlException) {
	        System.err.println("Could NOT Perform Query 3");
	        System.err.println(sqlException);
	    }
	   
	    try{
	        ResultSet rs = stat.executeQuery("select * from Artists LEFT OUTER JOIN CDs on Artists.ArtistID = CDs.ArtistID;");
		    System.out.println("\n\nselect Artists.ArtistName, CDs.Title from Artists LEFT OUTER JOIN CDs on Artists.ArtistID = CDs.ArtistID; = ");
	        QueryResult.print(rs, columnInfo1);

	        rs.close();
	    }catch(SQLException sqlException) {
	        System.err.println("Could NOT Perform Query 4");
	        System.err.println(sqlException);
	    }
	   
	    try{
	        ResultSet rs = stat.executeQuery("select Artists.ArtistID, Artists.ArtistName, CDs.CDID, CDs.ArtistID, CDs.Title, CDs.Date  from CDs LEFT OUTER JOIN Artists on Artists.ArtistID = CDs.ArtistID;");
		    System.out.println("\n\nHow to do a Right Outer Join in SQLite");
	        System.out.println("select Artists.ArtistName, CDs.Title from Artists LEFT OUTER JOIN CDs on Artists.ArtistID = CDs.ArtistID; = ");
	        QueryResult.print(rs, columnInfo1);

	        rs.close();
	    }catch(SQLException sqlException) {
	        System.err.println("Could NOT Perform Query 5");
	        System.err.println(sqlException);
	    }
	   
	    try{
	        ResultSet rs = stat.executeQuery("select Artists.ArtistID, Artists.ArtistName, CDs.CDID, CDs.ArtistID, CDs.Title, CDs.Date  from Artists LEFT OUTER JOIN CDs on Artists.ArtistID = CDs.ArtistID" +
	                                         " UNION " +
	                                         "select Artists.ArtistID, Artists.ArtistName, CDs.CDID, CDs.ArtistID, CDs.Title, CDs.Date  from CDs LEFT OUTER JOIN Artists on Artists.ArtistID = CDs.ArtistID;");
		    System.out.println("\n\nHow to do a Right Outer Join in SQLite");
	        System.out.println("select Artists.ArtistName, CDs.Title from Artists LEFT OUTER JOIN CDs on Artists.ArtistID = CDs.ArtistID; = ");
	        QueryResult.print(rs, columnInfo1);

	        rs.close();
	    }catch(SQLException sqlException) {
	        System.err.println("Could NOT Perform Query 5");
	        System.err.println(sqlException);
	    }
	   
	   
        connection.close();
    }
}

