package sqlite;

import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.PreparedStatement;
import java.sql.SQLException;
import java.sql.Statement;

public class SetUp3 {
	public static Connection initialize() throws Exception {
        Class.forName("org.sqlite.JDBC");
        Connection connection = DriverManager.getConnection("jdbc:sqlite:sample3.db");
		initializeArtist(connection);
		initializeCDs(connection);
		return connection;
	}

    private static void initializeArtist(Connection connection) {
        try{
            Statement stat = connection.createStatement();
            stat.executeUpdate("drop table if exists Artists;");
            stat.executeUpdate("create table Artists (ArtistID INTEGER PRIMARY KEY, ArtistName TEXT);");
            PreparedStatement prep = connection.prepareStatement( "insert into Artists values (?, ?);");
            prep.setString(1, "1");
            prep.setString(2, "Peter Gabriel");
            prep.addBatch();
            prep.setString(1, "2");
            prep.setString(2, "Bruce Hornsby");
            prep.addBatch();
            prep.setString(1, "3");
            prep.setString(2, "Lyle Lovett");
            prep.addBatch();
            prep.setString(1, "4");
            prep.setString(2, "Beach Boys");
            prep.addBatch();
            prep.setString(1, "5");
            prep.setString(2, "Supernatural");
            prep.addBatch();

            connection.setAutoCommit(false);
            prep.executeBatch();
            connection.setAutoCommit(true);
        }catch(SQLException sqlException) {
            System.err.println("Could NOT initialize Artists");
        }
    }

    private static void initializeCDs(Connection connection) {
        try{
            Statement stat = connection.createStatement();
            stat.executeUpdate("drop table if exists CDs;");
            stat.executeUpdate("create table CDs (CDID INTEGER PRIMARY KEY, ArtistID INTEGER REFERENCES Artist(ArtistID), Title TEXT, Date TEXT);");
            stat.execute("PRAGMA foreign_keys=ON;");
            PreparedStatement prep = connection.prepareStatement( "insert into CDs values (?, ?, ?, ?);");
            prep.setString(1, "1");
            prep.setString(2, "1");
            prep.setString(3, "So");
            prep.setString(4, "1984");
            prep.addBatch();
            prep.setString(1, "2");
            prep.setString(2, "1");
            prep.setString(3, "Us");
            prep.setString(4, "1992");
            prep.addBatch();
            prep.setString(1, "3");
            prep.setString(2, "2");
            prep.setString(3, "The Way It Is");
            prep.setString(4, "1986");
            prep.addBatch();
            prep.setString(1, "4");
            prep.setString(2, "2");
            prep.setString(3, "Scenes From The Southside");
            prep.setString(4, "1990");
            prep.addBatch();
            prep.setString(1, "5");
            prep.setString(2, "1");
            prep.setString(3, "Security");
            prep.setString(4, "1990");
            prep.addBatch();
            prep.setString(1, "6");
            prep.setString(2, "3");
            prep.setString(3, "Joshua Judges Ruth");
            prep.setString(4, "1992");
            prep.addBatch();
            prep.setString(1, "7");
            prep.setString(2, "4");
            prep.setString(3, "Pet Sounds");
            prep.setString(4, "1966");
            prep.addBatch();
            prep.setString(1, "8");
            prep.setString(2, "6");
            prep.setString(3, "Good Vibes");
            prep.setString(4, "1968");
            prep.addBatch();

            connection.setAutoCommit(false);
            prep.executeBatch();
            connection.setAutoCommit(true);
        }catch(SQLException sqlException) {
            System.err.println("Could NOT initialize CDs");
        }
    }
}

