package sqlite;

import java.sql.Connection;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.ArrayList;

public class Table {
    public static void print(Connection connection, String tableName) {
    	System.out.println("TABLE: " + tableName);
    	for(int i = 0; i < tableName.length() + 7; i++) {
    		System.out.print("-");
    	}
    	System.out.println();
    	
    	int numberOfRows = getNumberOfRows(connection, tableName);
	    String[][] columnInfo = getColumnInfo(connection, tableName);
	    int numberOfColumns = columnInfo.length;
	    Object[][] tuples = new Object[numberOfRows][numberOfColumns];
	    try{
            Statement stat = connection.createStatement();
	        ResultSet rs = stat.executeQuery("select * from " + tableName + ";");
	    
	        int row = 0;
	        while (rs.next()) {
	        	for(int col = 1; col <= numberOfColumns; col++) { 
	        		tuples[row][col-1] = getTupleValue(rs, col, columnInfo[col-1][1]);
	        	}
	        	row++;
            }
	        rs.close();
	    }catch(SQLException sqlException) {
	        System.err.println("Could NOT get column names for " + tableName);
	    }

	    String[][] stringValues = getStringValues(numberOfRows, numberOfColumns, tuples, columnInfo);
	    int[] maxWidths = getMaxWidths(numberOfRows, numberOfColumns, columnInfo, stringValues);
	    
	    
	    for(int col = 0; col < numberOfColumns; col++) {
	    	System.out.print(padLeft(columnInfo[col][0], maxWidths[col]) + " ");
	    }
	    System.out.println();
	    for(int col = 0; col < numberOfColumns; col++) {
	    	for(int i = 0; i < maxWidths[col]; i++) {
	    		System.out.print("-");
	    	}
	    	System.out.print(" ");
	    }
	    System.out.println();
	    
	    for(int row = 0; row < numberOfRows; row++) {
	    	for(int col = 0; col < numberOfColumns; col++) {
	    		System.out.print(padLeft(stringValues[row][col], maxWidths[col]) + " ");
	    	}
	    	System.out.println();
	    }
    }
    
    private static String padLeft(String str, int width) {
    	
    	int numberOfSpaces = 0;
    	if(str == null) {
    		numberOfSpaces = width - 4;
    	} else {
    		numberOfSpaces = width - str.length();
    	}
    	
    	StringBuilder result = new StringBuilder();
    	for(int i = 0; i < numberOfSpaces; i++) {
    		result.append(' ');
    	}
    	
    	if(str == null) {
    		result.append("NULL");
    	} else {
    		result.append(str);
    	}
    	
    	return result.toString();
    }
    
    private static int[] getMaxWidths(int numberOfRows, int numberOfColumns, String[][] columnInfo, String[][] stringValues) {
    	int[] result = new int[columnInfo.length];
    	for(int col = 0; col < numberOfColumns; col++) {
    		result[col] = columnInfo[col][0].length();
    	}
    	
    	for(int row = 0; row < numberOfRows; row++) {
    		for(int col = 0; col < numberOfColumns; col++) {
    			int width = 0;
    			if(stringValues[row][col] == null) {
    				width = 4;
    			} else {
    			    width = stringValues[row][col].length();
    			}
    			
    			if(result[col] < width) {
    				result[col] = width;
    			}
    		}
    	}
    
    	return result;
    }
    
    private static String[][] getStringValues(int rows, int cols, Object[][] tuples, String[][] columnInfo) {
    	String[][] result = new String[rows][cols];
    	for(int row = 0; row < rows; row++) {
    		for(int col = 0; col < cols; col++) {
    			switch(columnInfo[col][1]) {
    	       		case "INTEGER":
            		    result[row][col] = Integer.toString((Integer)tuples[row][col]);
        			    break;
        		    case "TEXT":
            		    result[row][col] = (String)tuples[row][col];
        			    break;
        		    case "REAL":
            		    result[row][col] = Double.toString((Double)tuples[row][col]);
        			    break;
        		    case "NULL":
            		    result[row][col] = "NULL";
        			    break;
        		    case "BLOB":
            		    result[row][col] = "UNKNOWN";
        			    break;
        	    }    			
    		}
    	}
    	return result;
    	
    }
    
    private static Object getTupleValue(ResultSet rs, int column, String type) {
    	Object result = null;
    	try{
    	switch(type) {
    		case "INTEGER":
        		result = (Object)rs.getInt(column);
    			break;
    		case "TEXT":
        		result = (Object)rs.getString(column);
    			break;
    		case "REAL":
        		result = (Object)rs.getDouble(column);
    			break;
    		case "NULL":
    			break;
    		case "BLOB":
        		result = null;
    			break;
    	}
    	}catch(SQLException e) {
    		System.err.println("ERROR which trying to get value of type " + type + " from column " + column);
    	}
    	return result;
    }
  
    private static String[][] getColumnInfo(Connection connection, String tableName) {
	    ArrayList<String> names = new ArrayList<String>();
	    ArrayList<String> types = new ArrayList<String>();
	    try{
            Statement stat = connection.createStatement();
	        ResultSet rs = stat.executeQuery("PRAGMA table_info(" + tableName + ");");
	    
	        while (rs.next()) {
	            names.add(rs.getString("name"));
	            types.add(rs.getString("type"));
            }
	        rs.close();
	    }catch(SQLException sqlException) {
	        System.err.println("Could NOT get column names for " + tableName);
	    }
	  
	    String[][] result = new String[names.size()][2];
	    for(int i = 0; i < names.size(); i++) {
		    result[i][0] = names.get(i);
		    result[i][1] = types.get(i);
	    }
	  
	    return result;
    }
  
    private static int getNumberOfRows(Connection connection, String tableName) {
	    int result = -1;
	    try{
            Statement stat = connection.createStatement();
	        ResultSet rs = stat.executeQuery("select count(*) from " + tableName + ";");
	        result = rs.getInt(1);

	        rs.close();
	    }catch(SQLException sqlException) {
	        System.err.println("Could NOT get number of columns from " + tableName);
	    }
	  
	    return result;
    }
}

